@@ -57,8 +57,8 @@ def filter_validity(arg="validity", **kwargs):
     validity = kwargs.get(arg)
     if validity is None:
         return (
-            Q(validity_from__lte=core.datetime.datetime.now()),
             Q(legacy_id__isnull=True),
+            Q(validity_to__isnull=True),
         )
     return (
        Q(validity_from__lte=validity),
